#!/usr/bin/env python
# SPDX-License-Identifier: ISC
#
# Copyright (c) 2022 by VMware, Inc. ("VMware")
#                       Shreenidhi A R <rshreenidhi@vmware.com>
# Used Copyright (c) 2018 by Network Device Education Foundation, Inc. ("NetDEF")
# in this file.
#
"""
Following tests are covered.
1. Verify default-originate route with default static and network command
2. Verify default-originate route with aggregate summary command
3. Verfiy  default-originate behaviour in ecmp
"""
import os
import sys
import time
import pytest
import datetime
from lib.topolog import logger
from time import sleep

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib.topogen import Topogen, get_topogen
from lib.topojson import build_config_from_json
from lib.topolog import logger
from lib import topotest

from lib.bgp import (
    verify_bgp_convergence,
    create_router_bgp,
    verify_bgp_rib,
    get_dut_as_number,
    verify_rib_default_route,
    verify_fib_default_route,
)
from lib.common_config import (
    verify_fib_routes,
    step,
    create_prefix_lists,
    run_frr_cmd,
    create_route_maps,
    shutdown_bringup_interface,
    get_frr_ipv6_linklocal,
    start_topology,
    apply_raw_config,
    write_test_header,
    check_address_types,
    write_test_footer,
    reset_config_on_routers,
    create_static_routes,
    check_router_status,
)

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))
sys.path.append(os.path.join(CWD, "../lib/"))

# Required to instantiate the topology builder class.

# pylint: disable=C0413
# Import topogen and topotest helpers

# Global variables
topo = None
NETWORK1_1 = {"ipv4": "198.51.1.1/32", "ipv6": "2001:DB8::1:1/128"}
NETWORK1_2 = {"ipv4": "198.51.1.2/32", "ipv6": "2001:DB8::1:2/128"}
NETWORK1_3 = {"ipv4": "198.51.1.3/32", "ipv6": "2001:DB8::1:3/128"}
NETWORK1_4 = {"ipv4": "198.51.1.4/32", "ipv6": "2001:DB8::1:4/128"}
NETWORK1_5 = {"ipv4": "198.51.1.5/32", "ipv6": "2001:DB8::1:5/128"}

ipv4_uptime_dict = {
    "r2": {
        "static_routes": [
            {"network": "0.0.0.0/0"},
        ]
    }
}

ipv6_uptime_dict = {
    "r2": {
        "static_routes": [
            {"network": "::/0"},
        ]
    }
}

DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
NEXT_HOP_IP = {"ipv4": "Null0", "ipv6": "Null0"}

pytestmark = [pytest.mark.bgpd]


def setup_module(mod):
    """
    Sets up the pytest environment

    * `mod`: module name
    """
    testsuite_run_time = time.asctime(time.localtime(time.time()))
    logger.info("Testsuite start time: {}".format(testsuite_run_time))
    logger.info("=" * 40)

    logger.info("Running setup_module to create topology")

    # This function initiates the topology build with Topogen...
    json_file = "{}/bgp_default_originate_2links.json".format(CWD)
    tgen = Topogen(json_file, mod.__name__)
    global topo
    topo = tgen.json_topo
    # ... and here it calls Mininet initialization functions.

    # Starting topology, create tmp files which are loaded to routers
    #  to start daemons and then start routers
    start_topology(tgen)

    # Creating configuration from JSON
    build_config_from_json(tgen, topo)

    global ADDR_TYPES
    global BGP_CONVERGENCE
    global DEFAULT_ROUTES
    global DEFAULT_ROUTE_NXT_HOP_LINK1, DEFAULT_ROUTE_NXT_HOP_LINK2
    ADDR_TYPES = check_address_types()
    BGP_CONVERGENCE = verify_bgp_convergence(tgen, topo)
    assert BGP_CONVERGENCE is True, "setup_module :Failed \n Error: {}".format(
        BGP_CONVERGENCE
    )

    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}

    interface = topo["routers"]["r1"]["links"]["r2-link1"]["interface"]
    ipv6_link_local = get_frr_ipv6_linklocal(tgen, "r1", intf=interface)
    ipv4_nxt_hop = topo["routers"]["r1"]["links"]["r2-link1"]["ipv4"].split("/")[0]
    ipv6_nxt_hop = topo["routers"]["r1"]["links"]["r2-link1"]["ipv6"].split("/")[0]
    DEFAULT_ROUTE_NXT_HOP_LINK1 = {"ipv4": ipv4_nxt_hop, "ipv6": ipv6_link_local}

    interface = topo["routers"]["r1"]["links"]["r2-link2"]["interface"]
    ipv6_link_local = get_frr_ipv6_linklocal(tgen, "r1", intf=interface)
    ipv4_nxt_hop = topo["routers"]["r1"]["links"]["r2-link2"]["ipv4"].split("/")[0]
    ipv6_nxt_hop = topo["routers"]["r1"]["links"]["r2-link2"]["ipv6"].split("/")[0]
    DEFAULT_ROUTE_NXT_HOP_LINK2 = {"ipv4": ipv4_nxt_hop, "ipv6": ipv6_link_local}
    logger.info("Running setup_module() done")


def teardown_module():
    """Teardown the pytest environment"""

    logger.info("Running teardown_module to delete topology")

    tgen = get_topogen()

    # Stop toplogy and Remove tmp files
    tgen.stop_topology()

    logger.info(
        "Testsuite end time: {}".format(time.asctime(time.localtime(time.time())))
    )
    logger.info("=" * 40)


#####################################################
#
#                      Local API's
#
#####################################################


def get_rib_route_uptime(tgen, addr_type, dut, input_dict):
    """
    Verify route uptime in RIB using "show ip route"

    Parameters
    ----------
    * `tgen` : topogen object
    * `addr_type` : ip type, ipv4/ipv6
    * `dut`: Device Under Test, for which user wants to test the data
    * `input_dict` : input dict, has details of static routes
    * `route_uptime`: uptime of the routes

    Usage
    -----
    # Creating static routes for r1
     input_dict_r1 = {
            "r1": {
                "static_routes": [
                    {
                        "network": "147.10.13.4/32"
                    },
                   {
                       "network": "147.10.12.0/24"
                   },
                    {
                        "network": "147.10.13.4/32"
                    },
                   {
                       "network": "147.10.13.4/32"
                   },
                   {
                       "network": "147.10.13.4/32"
                   }
                ]
            }
    }


    Returns
    -------
    errormsg(str) or True
    """

    logger.info("Entering lib API: get_rib_route_uptime()")
    route_time = []
    router_list = tgen.routers()
    for routerInput in input_dict.keys():
        for router, rnode in router_list.items():
            if router != dut:
                continue

            logger.info("Checking router %s RIB:", router)

            # Verifying RIB routes
            if addr_type == "ipv4":
                command = "show ip route"
            else:
                command = "show ipv6 route"

            if "static_routes" in input_dict[routerInput]:
                static_routes = input_dict[routerInput]["static_routes"]

                for static_route in static_routes:
                    if "vrf" in static_route and static_route["vrf"] is not None:
                        logger.info(
                            "[DUT: {}]: Verifying routes for VRF:"
                            " {}".format(router, static_route["vrf"])
                        )
                        cmd = "{} vrf {}".format(command, static_route["vrf"])

                    else:
                        cmd = "{}".format(command)

                    cmd = "{} json".format(cmd)

                    rib_routes_json = run_frr_cmd(rnode, cmd, isjson=True)

                    if bool(rib_routes_json) is False:
                        errormsg = "No route found in rib of router {}..".format(router)
                        return errormsg
                    network = static_route["network"]
                    route_time.append(rib_routes_json[network][0]["uptime"])

    logger.info("Exiting lib API: get_rib_route_uptime()")
    return route_time


def verify_the_uptime(time_stamp_before, time_stamp_after, incremented=None):
    """
    time_stamp_before : string the time stamp captured
    time_stamp_after : string the time stamp captured
    """
    uptime_before = datetime.datetime.strptime(time_stamp_before[0], "%H:%M:%S")
    uptime_after = datetime.datetime.strptime(time_stamp_after[0], "%H:%M:%S")

    if incremented == True:
        if uptime_before < uptime_after:
            logger.info(
                "  The Uptime before [{}] is less than [{}].......PASSED ".format(
                    time_stamp_before, time_stamp_after
                )
            )
            return True
        else:
            logger.error(
                "  The Uptime before [{}] is greater than the uptime after [{}].......FAILED ".format(
                    time_stamp_before, time_stamp_after
                )
            )
            return False
    else:
        logger.info(
            "  The Uptime before [{}] the same as after [{}] ".format(
                time_stamp_before, time_stamp_after
            )
        )
        return True


def get_best_path_route_in_FIB(tgen, topo, dut, network):
    """
    API to verify the best route in FIB and return the ipv4 and ipv6 nexthop for the given route
    command
    =======
    show ip route
    show ipv6 route
    params
    ======
    dut : device under test :
    network ; route (ip) to which the best route to be retrieved
    Returns
    ========
    on success : return dict with next hops for the best hop
    on failure : return error message with boolean False
    """
    is_ipv4_best_path_found = False
    rnode = tgen.routers()[dut]
    ipv4_show_bgp_json = run_frr_cmd(rnode, "sh ip bgp  json ", isjson=True)
    ipv6_show_bgp_json = run_frr_cmd(
        rnode, "sh ip bgp ipv6 unicast  json ", isjson=True
    )
    output_dict = {"ipv4": None, "ipv6": None}
    ipv4_nxt_hop_count = len(ipv4_show_bgp_json["routes"][network["ipv4"]])
    for index in range(ipv4_nxt_hop_count):
        if "bestpath" in ipv4_show_bgp_json["routes"][network["ipv4"]][index].keys():
            best_path_ip = ipv4_show_bgp_json["routes"][network["ipv4"]][index][
                "nexthops"
            ][0]["ip"]
            output_dict["ipv4"] = best_path_ip
            logger.info(
                "[DUT [{}]] Best path for the route {} is  {} ".format(
                    dut, network["ipv4"], best_path_ip
                )
            )
            is_ipv4_best_path_found = True
        else:
            logger.error("ERROR....! No Best Path  Found in BGP RIB.... FAILED")

    ipv6_nxt_hop_count = len(ipv6_show_bgp_json["routes"][network["ipv6"]])
    for index in range(ipv6_nxt_hop_count):
        if "bestpath" in ipv6_show_bgp_json["routes"][network["ipv6"]][index].keys():
            ip_add_count = len(
                ipv6_show_bgp_json["routes"][network["ipv6"]][index]["nexthops"]
            )
            for i_index in range(ip_add_count):
                if (
                    "global"
                    in ipv6_show_bgp_json["routes"][network["ipv6"]][index]["nexthops"][
                        i_index
                    ]["scope"]
                ):
                    best_path_ip = ipv6_show_bgp_json["routes"][network["ipv6"]][index][
                        "nexthops"
                    ][i_index]["ip"]
                    output_dict["ipv6"] = best_path_ip
                    logger.info(
                        "[DUT [{}]] Best path for the route {} is  {} ".format(
                            dut, network["ipv6"], best_path_ip
                        )
                    )

        else:
            logger.error("ERROR....! No Best Path  Found in BGP RIB.... FAILED")
    if is_ipv4_best_path_found:
        return output_dict
    else:
        logger.error("ERROR...! Unable to find the Best Path in the RIB")
        return False


#####################################################
#
#                      Testcases
#
#####################################################


def test_verify_bgp_default_originate_with_default_static_route_p1(request):
    """
    Summary: "Verify default-originate route with default static and network command  "

    """
    tgen = get_topogen()
    global BGP_CONVERGENCE, DEFAULT_ROUTE_NXT_HOP_LINK1, DEFAULT_ROUTE_NXT_HOP_LINK2, DEFAULT_ROUTES

    if BGP_CONVERGENCE != True:
        pytest.skip("skipped because of BGP Convergence failure")
    # test case name
    tc_name = request.node.name
    write_test_header(tc_name)
    if tgen.routers_have_failure():
        check_router_status(tgen)
    reset_config_on_routers(tgen)

    step("Configure 2 link between R1 and R2")
    step("Configure IPV4 and IPV6 EBGP between R1 and R2  both the links")
    step("Configure default-originate on R1 IPv4 and IPv6 BGP session link-1 only ")
    local_as = get_dut_as_number(tgen, dut="r1")
    default_originate_config = {
        "r1": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "default_originate": {"r2": {"dest-link": "r1-link1"}}
                        }
                    },
                    "ipv6": {
                        "unicast": {
                            "default_originate": {"r2": {"dest-link": "r1-link1"}}
                        }
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, topo, default_originate_config)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Verify IPv4/IPv6 default originate routes present on R2 nexthop as link-1")
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r2": {
                "static_routes": [
                    {
                        "network": [DEFAULT_ROUTES[addr_type]],
                        "next_hop": DEFAULT_ROUTE_NXT_HOP_LINK1[addr_type],
                    }
                ]
            }
        }

        result = verify_fib_routes(
            tgen,
            addr_type,
            "r2",
            static_routes_input,
            next_hop=DEFAULT_ROUTE_NXT_HOP_LINK1[addr_type],
        )
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_bgp_rib(
            tgen,
            addr_type,
            "r2",
            static_routes_input,
            next_hop=DEFAULT_ROUTE_NXT_HOP_LINK1[addr_type],
        )
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Configure  network command on R1 (0.0.0.0/0 and 0::0/0) for IPv4 and IPv6 address family "
    )
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
    for addr_type in ADDR_TYPES:
        input_advertise = {
            "r1": {
                "bgp": {
                    "address_family": {
                        addr_type: {
                            "unicast": {
                                "advertise_networks": [
                                    {"network": [DEFAULT_ROUTES[addr_type]]}
                                ]
                            }
                        }
                    }
                }
            }
        }

        result = create_router_bgp(tgen, topo, input_advertise)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step("No change on IPv4/IPv6  default-originate route advertised from link1")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("verify 0.0.0.0/0 and 0::0/0 route also get advertised from link-2  ")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step(
        "Before removing default originate from R1 link -1 IPv4 and IPv6 address family taking the uptime snapshot"
    )
    uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step("Remove default originate from R1 link -1 IPv4 and IPv6 address family ")
    local_as = get_dut_as_number(tgen, dut="r1")
    default_originate_config = {
        "r1": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "default_originate": {
                                "r2": {"dest-link": "r1-link1", "delete": True}
                            }
                        }
                    },
                    "ipv6": {
                        "unicast": {
                            "default_originate": {
                                "r2": {"dest-link": "r1-link1", "delete": True}
                            }
                        }
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, topo, default_originate_config)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Routes must be learned from network command")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("After removing the default originate  on R1 taking the uptime snapshot")
    uptime_after_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_after_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step(
        "After removing the default-originate uptime should get reset for link-1 learn route"
    )
    result = verify_the_uptime(uptime_before_ipv4, uptime_after_ipv4, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_the_uptime(uptime_before_ipv6, uptime_after_ipv6, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Taking uptime snapshot before configuring default - originate")
    uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)
    sleep(1)

    step(
        "Configure default-originate on R1 link-1 again for IPv4 and IPv6 address family"
    )
    local_as = get_dut_as_number(tgen, dut="r1")
    default_originate_config = {
        "r1": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "default_originate": {
                                "r2": {
                                    "dest-link": "r1-link1",
                                }
                            }
                        }
                    },
                    "ipv6": {
                        "unicast": {
                            "default_originate": {
                                "r2": {
                                    "dest-link": "r1-link1",
                                }
                            }
                        }
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, topo, default_originate_config)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Verify No change on R2 routing and BGP table for both the links ")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Taking snapshot after configuring default - originate")
    uptime_after_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_after_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step(
        "After configuring the default-originate uptime should not get reset for link-1 learn route"
    )
    result = verify_the_uptime(uptime_before_ipv4, uptime_after_ipv4, incremented=True)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_the_uptime(uptime_before_ipv6, uptime_after_ipv6, incremented=True)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Taking uptime snapshot before  removing  network 0.0.0.0  ")
    uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step("Remove network command from R1 IPv4/IPv6 address family ")
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
    for addr_type in ADDR_TYPES:
        input_advertise = {
            "r1": {
                "bgp": {
                    "address_family": {
                        addr_type: {
                            "unicast": {
                                "advertise_networks": [
                                    {
                                        "network": [DEFAULT_ROUTES[addr_type]],
                                        "delete": True,
                                    }
                                ]
                            }
                        }
                    }
                }
            }
        }

        result = create_router_bgp(tgen, topo, input_advertise)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Verify 0.0.0.0/0 and 0::0/0 route get removed from link-2  and default-originate IPv4/IPv6 route learn on link-1"
    )
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
        expected=False,
    )
    assert (
        result is not True
    ), "Testcase {} : Failed \n Route from link2 is not expected \n Error: {}".format(
        tc_name, result
    )
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
        expected=False,
    )
    assert (
        result is not True
    ), "Testcase {} : Failed\n Route from link2 is not expected \n  Error: {}".format(
        tc_name, result
    )

    uptime_after_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_after_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step(
        "After removing default originate command on R1 verify that the uptime got reset on R2"
    )

    result = verify_the_uptime(uptime_before_ipv4, uptime_after_ipv4, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_the_uptime(uptime_before_ipv6, uptime_after_ipv6, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Taking uptime snapshot before  configuring static route  network")
    uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step(
        "Configure static default route for IPv4 and IPv6 (0.0.0.0/0 next-hop Null0 and 0::0/0 next-hop Null0) on R1"
    )
    static_routes_input = {
        "r1": {
            "static_routes": [
                {
                    "network": "0.0.0.0/0",
                    "next_hop": NEXT_HOP_IP["ipv4"],
                },
                {
                    "network": "0::0/0",
                    "next_hop": NEXT_HOP_IP["ipv6"],
                },
            ]
        }
    }
    result = create_static_routes(tgen, static_routes_input)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("verifyIPv4 and IPv6 static routes are configure and up on R1 ")
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r1": {
                "static_routes": [
                    {
                        "network": "0.0.0.0/0",
                        "next_hop": NEXT_HOP_IP["ipv4"],
                    },
                    {
                        "network": "0::0/0",
                        "next_hop": NEXT_HOP_IP["ipv6"],
                    },
                ]
            }
        }
        result = verify_fib_routes(tgen, addr_type, "r1", static_routes_input)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step("Configure redistribute static on IPv4 and IPv6 address family")
    redistribute_static = {
        "r1": {
            "bgp": {
                "address_family": {
                    "ipv4": {"unicast": {"redistribute": [{"redist_type": "static"}]}},
                    "ipv6": {"unicast": {"redistribute": [{"redist_type": "static"}]}},
                }
            }
        }
    }
    result = create_router_bgp(tgen, topo, redistribute_static)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Verify No change on IPv4/IPv6  default-originate route advertised from link1")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("verify 0.0.0.0/0 and 0::0/0 route also get advertised from link-2 ")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
        expected=False,
    )
    assert (
        result is not True
    ), "Testcase {} : Failed\n Best Path sould be advertised in routes\n Error: {}".format(
        tc_name, result
    )

    step("Taking uptime snapshot before  removing  default originate")
    uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step("Remove default-originate from link-1 from IPv4 and IPv6 neighbor ")
    local_as = get_dut_as_number(tgen, dut="r1")
    default_originate_config = {
        "r1": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "default_originate": {
                                "r2": {"dest-link": "r1-link1", "delete": True}
                            }
                        }
                    },
                    "ipv6": {
                        "unicast": {
                            "default_originate": {
                                "r2": {"dest-link": "r1-link1", "delete": True}
                            }
                        }
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, topo, default_originate_config)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Taking uptime snapshot after  removing  default originate")
    uptime_after_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_after_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step("verify the up time , up time should get reset ")
    result = verify_the_uptime(uptime_before_ipv4, uptime_after_ipv4, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_the_uptime(uptime_before_ipv6, uptime_after_ipv6, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Verify No change on IPv4/IPv6  default-originate route advertised from link1")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Taking uptime snapshot before  configuring  default originate")
    uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step(
        " Configure default-originate on link-1 again for IPv4 and IPv6 address family"
    )
    local_as = get_dut_as_number(tgen, dut="r1")
    default_originate_config = {
        "r1": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "default_originate": {
                                "r2": {
                                    "dest-link": "r1-link1",
                                }
                            }
                        }
                    },
                    "ipv6": {
                        "unicast": {
                            "default_originate": {
                                "r2": {
                                    "dest-link": "r1-link1",
                                }
                            }
                        }
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, topo, default_originate_config)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Verify No change on IPv4/IPv6  default-originate route advertised from link1")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
        expected=False,
    )
    assert result is not True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Taking uptime snapshot after  configuring  default originate")
    uptime_after_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_after_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step("After configuring the default originate the uptime should not get reset ")
    result = verify_the_uptime(uptime_before_ipv4, uptime_after_ipv4, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_the_uptime(uptime_before_ipv6, uptime_after_ipv6, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Taking uptime snapshot before removing redistribute static")
    uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)
    sleep(1)

    step("Remove redistribute static from IPv4 and IPv6 address family ")
    input_dict_1 = {
        "r1": {
            "bgp": {
                "local_as": get_dut_as_number(tgen, dut="r1"),
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "redistribute": [{"redist_type": "static", "delete": True}]
                        }
                    },
                    "ipv6": {
                        "unicast": {
                            "redistribute": [{"redist_type": "static", "delete": True}]
                        }
                    },
                },
            }
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_1)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify No change on IPv4/IPv6  default-originate route advertised from link1")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Now look that the route is not pointed at link2")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
        expected=False,
    )
    assert result is not True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
        expected=False,
    )
    assert result is not True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Taking uptime snapshot after removing redistribute static")
    uptime_after_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_after_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step("After removing default originate the route uptime should get reset ")
    result = verify_the_uptime(uptime_before_ipv4, uptime_after_ipv4, incremented=True)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_the_uptime(uptime_before_ipv6, uptime_after_ipv6, incremented=True)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    write_test_footer(tc_name)


def test_verify_bgp_default_originate_with_aggregate_summary_p1(request):
    """
    Summary: "Verify default-originate route with aggregate summary command"
    """
    tgen = get_topogen()
    global BGP_CONVERGENCE
    tc_name = request.node.name
    write_test_header(tc_name)
    if tgen.routers_have_failure():
        check_router_status(tgen)
    reset_config_on_routers(tgen)
    if BGP_CONVERGENCE != True:
        pytest.skip("skipped because of BGP Convergence failure")

    step("After changing the BGP AS Path Verify the BGP Convergence")
    BGP_CONVERGENCE = verify_bgp_convergence(tgen, topo)
    assert BGP_CONVERGENCE is True, "setup_module :Failed \n Error: {}".format(
        BGP_CONVERGENCE
    )

    step("Configure default-originate on R1 IPv4 and IPv6 BGP session link-1 only")
    local_as = get_dut_as_number(tgen, dut="r1")
    default_originate_config = {
        "r1": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "default_originate": {"r2": {"dest-link": "r1-link1"}}
                        }
                    },
                    "ipv6": {
                        "unicast": {
                            "default_originate": {"r2": {"dest-link": "r1-link1"}}
                        }
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, topo, default_originate_config)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify IPv4/IPv6 default originate routes present on R2 nexthop as link-1,on R2"
    )
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Configure 5 static  route for IPv4 and IPv6  on R0")
    for addr_type in ADDR_TYPES:
        input_advertise = {
            "r1": {
                "bgp": {
                    "address_family": {
                        addr_type: {
                            "unicast": {
                                "advertise_networks": [
                                    {
                                        "network": [NETWORK1_1[addr_type]],
                                        "next_hop": NEXT_HOP_IP[addr_type],
                                    },
                                    {
                                        "network": [NETWORK1_2[addr_type]],
                                        "next_hop": NEXT_HOP_IP[addr_type],
                                    },
                                    {
                                        "network": [NETWORK1_3[addr_type]],
                                        "next_hop": NEXT_HOP_IP[addr_type],
                                    },
                                    {
                                        "network": [NETWORK1_4[addr_type]],
                                        "next_hop": NEXT_HOP_IP[addr_type],
                                    },
                                    {
                                        "network": [NETWORK1_5[addr_type]],
                                        "next_hop": NEXT_HOP_IP[addr_type],
                                    },
                                ]
                            }
                        }
                    }
                }
            }
        }

        result = create_router_bgp(tgen, topo, input_advertise)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step("Before configuring the aggregate route taking uptime snapshot  ")
    uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step("Configure aggregate summary command  for IPv4 and IPv6 address family ")
    local_as = get_dut_as_number(tgen, dut="r1")
    raw_config = {
        "r1": {
            "raw_config": [
                "router bgp {}".format(local_as),
                "address-family ipv4 unicast",
                "aggregate-address {} summary-only".format("0.0.0.0/0 "),
                "exit-address-family",
                "address-family ipv6 unicast",
                "aggregate-address {} summary-only".format("0::0/0"),
                "exit-address-family",
            ]
        }
    }
    result = apply_raw_config(tgen, raw_config)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step(
        "verify that no change on IPv4/IPv6  default-originate route advertised from link1  0.0.0.0/0 and 0::0/0 route also get advertised from link-2    on R2"
    )
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("After configuring the aggregate route taking uptime snapshot  ")
    uptime_after_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_after_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step(
        "After Configuring  the aggregate route uptime should get reset for link-1 learn route"
    )
    result = verify_the_uptime(uptime_before_ipv4, uptime_after_ipv4, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_the_uptime(uptime_before_ipv6, uptime_after_ipv6, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Before removing default originate  taking uptime snapshot ")
    uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step("Remove default originate from R1 link -1 IPv4 and IPv6 address family")
    local_as = get_dut_as_number(tgen, dut="r1")
    default_originate_config = {
        "r1": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "default_originate": {
                                "r2": {"dest-link": "r1-link1", "delete": True}
                            }
                        }
                    },
                    "ipv6": {
                        "unicast": {
                            "default_originate": {
                                "r2": {"dest-link": "r1-link1", "delete": True}
                            }
                        }
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, topo, default_originate_config)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "verify that no change on IPv4/IPv6  default-originate route advertised from link1  0.0.0.0/0 and 0::0/0 route also get advertised from link-2    on R2"
    )
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("After removing default origin taking uptime snapshot  ")
    uptime_after_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_after_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step(
        "After removing the default-originate uptime should get reset for link-1 learn route"
    )
    result = verify_the_uptime(uptime_before_ipv4, uptime_after_ipv4, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_the_uptime(uptime_before_ipv6, uptime_after_ipv6, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Before Configuring default origin taking uptime snapshot ")
    uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step(
        "Configure default-originate on R1 link-1 again for IPv4 and IPv6 address family"
    )
    local_as = get_dut_as_number(tgen, dut="r1")
    default_originate_config = {
        "r1": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "default_originate": {
                                "r2": {
                                    "dest-link": "r1-link1",
                                }
                            }
                        }
                    },
                    "ipv6": {
                        "unicast": {
                            "default_originate": {
                                "r2": {
                                    "dest-link": "r1-link1",
                                }
                            }
                        }
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, topo, default_originate_config)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("After Configuring  default originate  taking uptime snapshot")
    uptime_after_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_after_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step(
        "After Configuring  the default-originate uptime should get reset for link-1 learn route"
    )
    result = verify_the_uptime(uptime_before_ipv4, uptime_after_ipv4, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_the_uptime(uptime_before_ipv6, uptime_after_ipv6, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Before removing aggregate -summary command taking the uptime snapshot ")
    uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step("remove aggregate summary command  for IPv4 and IPv6 address family ")
    local_as = get_dut_as_number(tgen, dut="r1")
    raw_config = {
        "r1": {
            "raw_config": [
                "router bgp {}".format(local_as),
                "address-family ipv4 unicast",
                "no aggregate-address {} summary-only".format("0.0.0.0/0"),
                "exit-address-family",
                "address-family ipv6 unicast",
                "no aggregate-address {} summary-only".format("0::0/0"),
                "exit-address-family",
            ]
        }
    }
    result = apply_raw_config(tgen, raw_config)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Verify Default-originate IPv4/IPv6 route learn on link-1 ")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK1,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Verify 0.0.0.0/0 and 0::0/0 route get removed from link-2 ")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
        expected=False,
    )
    assert result is not True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_LINK2,
        expected=False,
    )
    assert result is not True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("After removing aggregate -summary command taking the uptime snapshot ")
    uptime_after_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
    uptime_after_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)

    step("After removing aggregate command uptime should get reset ")
    result = verify_the_uptime(uptime_before_ipv4, uptime_after_ipv4, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    result = verify_the_uptime(uptime_before_ipv6, uptime_after_ipv6, incremented=False)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    write_test_footer(tc_name)


def test_verify_default_originate_with_2way_ecmp_p2(request):
    """
    Summary: "Verify default-originate route with 3 way ECMP and traffic "
    """

    tgen = get_topogen()
    global BGP_CONVERGENCE
    global DEFAULT_ROUTES
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}

    if BGP_CONVERGENCE != True:
        pytest.skip("skipped because of BGP Convergence failure")
    # test case name
    tc_name = request.node.name
    write_test_header(tc_name)
    if tgen.routers_have_failure():
        check_router_status(tgen)
    reset_config_on_routers(tgen)

    step("Populating next-hops details")
    r1_r2_ipv4_neighbor_ips = []
    r1_r2_ipv6_neighbor_ips = []
    r1_link = None
    for index in range(1, 3):
        r1_link = "r1-link" + str(index)
        r1_r2_ipv4_neighbor_ips.append(
            topo["routers"]["r2"]["links"][r1_link]["ipv4"].split("/")[0]
        )
        r1_r2_ipv6_neighbor_ips.append(
            topo["routers"]["r2"]["links"][r1_link]["ipv6"].split("/")[0]
        )

    step(
        "Configure default-originate on R1 for all the neighbor of IPv4 and IPv6 peers "
    )
    local_as = get_dut_as_number(tgen, dut="r1")
    for index in range(2):
        raw_config = {
            "r1": {
                "raw_config": [
                    "router bgp {}".format(local_as),
                    "address-family ipv4 unicast",
                    "neighbor {} default-originate".format(
                        r1_r2_ipv4_neighbor_ips[index]
                    ),
                    "exit-address-family",
                    "address-family ipv6 unicast",
                    "neighbor {} default-originate ".format(
                        r1_r2_ipv6_neighbor_ips[index]
                    ),
                    "exit-address-family",
                ]
            }
        }
        result = apply_raw_config(tgen, raw_config)
        assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step(
        "After configuring default-originate command , verify default  routes are advertised on R2 "
    )

    r2_link = None
    for index in range(1, 3):
        r2_link = "r2-link" + str(index)
        ipv4_nxt_hop = topo["routers"]["r1"]["links"][r2_link]["ipv4"].split("/")[0]
        interface = topo["routers"]["r1"]["links"][r2_link]["interface"]
        ipv6_link_local_nxt_hop = get_frr_ipv6_linklocal(tgen, "r1", intf=interface)
        DEFAULT_ROUTE_NXT_HOP = {"ipv4": ipv4_nxt_hop, "ipv6": ipv6_link_local_nxt_hop}

        result = verify_rib_default_route(
            tgen,
            topo,
            dut="r2",
            routes=DEFAULT_ROUTES,
            expected_nexthop=DEFAULT_ROUTE_NXT_HOP,
        )
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step("Ping R1 configure IPv4 and IPv6 loopback address from R2")
    pingaddr = topo["routers"]["r1"]["links"]["lo"]["ipv4"].split("/")[0]
    router = tgen.gears["r2"]

    def ping_router():
        output = router.run("ping -c 4 -w 4 {}".format(pingaddr))
        logger.info(output)
        if " 0% packet loss" not in output:
            return False

    _, res = topotest.run_and_expect(ping_router, None, count=10, wait=1)
    logger.info("Ping from R1 to R2 ... success")

    step("Shuting up the active route")
    network = {"ipv4": "0.0.0.0/0", "ipv6": "::/0"}
    ipv_dict = get_best_path_route_in_FIB(tgen, topo, dut="r2", network=network)
    dut_links = topo["routers"]["r1"]["links"]
    active_interface = None
    for key, _ in dut_links.items():
        ipv4_address = dut_links[key]["ipv4"].split("/")[0]
        ipv6_address = dut_links[key]["ipv6"].split("/")[0]
        if ipv_dict["ipv4"] == ipv4_address and ipv_dict["ipv6"] == ipv6_address:
            active_interface = dut_links[key]["interface"]

    logger.info(
        "Shutting down the interface {} on router {} ".format(active_interface, "r1")
    )
    shutdown_bringup_interface(tgen, "r1", active_interface, False)

    step("Verify the complete convergence  to fail after  shutting  the interface")
    result = verify_bgp_convergence(tgen, topo, expected=False)
    assert (
        result is not True
    ), " Testcase {} : After shuting down the interface  Convergence is expected to be Failed".format(
        tc_name
    )

    step(
        "Verify  routes from active best path is not received from  r1 after  shuting the interface"
    )
    r2_link = None
    for index in range(1, 3):
        r2_link = "r2-link" + str(index)
        ipv4_nxt_hop = topo["routers"]["r1"]["links"][r2_link]["ipv4"].split("/")[0]
        interface = topo["routers"]["r1"]["links"][r2_link]["interface"]
        ipv6_link_local_nxt_hop = get_frr_ipv6_linklocal(tgen, "r1", intf=interface)
        DEFAULT_ROUTE_NXT_HOP = {"ipv4": ipv4_nxt_hop, "ipv6": ipv6_link_local_nxt_hop}
        if index == 1:
            result = verify_rib_default_route(
                tgen,
                topo,
                dut="r2",
                routes=DEFAULT_ROUTES,
                expected_nexthop=DEFAULT_ROUTE_NXT_HOP,
                expected=False,
            )
            assert result is not True, "Testcase {} : Failed \n Error: {}".format(
                tc_name, result
            )
        else:
            result = verify_rib_default_route(
                tgen,
                topo,
                dut="r2",
                routes=DEFAULT_ROUTES,
                expected_nexthop=DEFAULT_ROUTE_NXT_HOP,
            )
            assert result is True, "Testcase {} : Failed \n Error: {}".format(
                tc_name, result
            )

    step("Ping R1 configure IPv4 and IPv6 loopback address from R2")
    pingaddr = topo["routers"]["r1"]["links"]["lo"]["ipv4"].split("/")[0]
    router = tgen.gears["r2"]
    output = router.run("ping -c 4 -w 4 {}".format(pingaddr))
    assert " 0% packet loss" in output, "Ping R1->R2  FAILED"
    logger.info("Ping from R1 to R2 ... success")

    step("No Shuting up the active route")

    shutdown_bringup_interface(tgen, "r1", active_interface, True)

    step("Verify the complete convergence after bringup the interface")
    result = verify_bgp_convergence(tgen, topo)
    assert (
        result is True
    ), " Testcase {} : After bringing up  the interface  complete convergence is expected ".format(
        tc_name
    )

    step("Verify all the routes are received from  r1 after no shuting the interface")
    r2_link = None
    for index in range(1, 3):
        r2_link = "r2-link" + str(index)
        ipv4_nxt_hop = topo["routers"]["r1"]["links"][r2_link]["ipv4"].split("/")[0]
        interface = topo["routers"]["r1"]["links"][r2_link]["interface"]
        ipv6_link_local_nxt_hop = get_frr_ipv6_linklocal(tgen, "r1", intf=interface)
        DEFAULT_ROUTE_NXT_HOP = {"ipv4": ipv4_nxt_hop, "ipv6": ipv6_link_local_nxt_hop}
        if index == 1:
            result = verify_rib_default_route(
                tgen,
                topo,
                dut="r2",
                routes=DEFAULT_ROUTES,
                expected_nexthop=DEFAULT_ROUTE_NXT_HOP,
            )
            assert result is True, "Testcase {} : Failed \n Error: {}".format(
                tc_name, result
            )
        else:
            result = verify_rib_default_route(
                tgen,
                topo,
                dut="r2",
                routes=DEFAULT_ROUTES,
                expected_nexthop=DEFAULT_ROUTE_NXT_HOP,
            )
            assert result is True, "Testcase {} : Failed \n Error: {}".format(
                tc_name, result
            )

    step(
        "Configure IPv4 and IPv6  route-map with deny option on R2 to filter default route  0.0.0.0/0 and 0::0/0"
    )
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
    input_dict_3 = {
        "r2": {
            "prefix_lists": {
                "ipv4": {
                    "Pv4": [
                        {
                            "seqid": "1",
                            "network": DEFAULT_ROUTES["ipv4"],
                            "action": "permit",
                        }
                    ]
                },
                "ipv6": {
                    "Pv6": [
                        {
                            "seqid": "1",
                            "network": DEFAULT_ROUTES["ipv6"],
                            "action": "permit",
                        }
                    ]
                },
            }
        }
    }
    result = create_prefix_lists(tgen, input_dict_3)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    input_dict_3 = {
        "r2": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "deny",
                        "seq_id": "1",
                        "match": {"ipv4": {"prefix_lists": "Pv4"}},
                    },
                ],
                "RMv6": [
                    {
                        "action": "deny",
                        "seq_id": "1",
                        "match": {"ipv6": {"prefix_lists": "Pv6"}},
                    },
                ],
            }
        }
    }
    result = create_route_maps(tgen, input_dict_3)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Apply route-map IN direction of R2 ( R2-R1) for IPv4 and IPv6 BGP neighbors")
    r2_link = None
    for index in range(1, 3):
        r2_link = "r2-link" + str(index)
        input_dict_4 = {
            "r2": {
                "bgp": {
                    "address_family": {
                        "ipv4": {
                            "unicast": {
                                "neighbor": {
                                    "r1": {
                                        "dest_link": {
                                            r2_link: {
                                                "route_maps": [
                                                    {"name": "RMv4", "direction": "in"}
                                                ]
                                            },
                                        }
                                    }
                                }
                            }
                        },
                        "ipv6": {
                            "unicast": {
                                "neighbor": {
                                    "r1": {
                                        "dest_link": {
                                            r2_link: {
                                                "route_maps": [
                                                    {"name": "RMv6", "direction": "in"}
                                                ]
                                            },
                                        }
                                    }
                                }
                            }
                        },
                    }
                }
            }
        }
        result = create_router_bgp(tgen, topo, input_dict_4)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step("After applying the route-map the routes are not expected in RIB ")
    r2_link = None
    for index in range(1, 3):
        r2_link = "r2-link" + str(index)
        ipv4_nxt_hop = topo["routers"]["r1"]["links"][r2_link]["ipv4"].split("/")[0]
        interface = topo["routers"]["r1"]["links"][r2_link]["interface"]
        ipv6_link_local_nxt_hop = get_frr_ipv6_linklocal(tgen, "r1", intf=interface)
        DEFAULT_ROUTE_NXT_HOP = {"ipv4": ipv4_nxt_hop, "ipv6": ipv6_link_local_nxt_hop}

        result = verify_rib_default_route(
            tgen,
            topo,
            dut="r2",
            routes=DEFAULT_ROUTES,
            expected_nexthop=DEFAULT_ROUTE_NXT_HOP,
            expected=False,
        )
        assert result is not True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    write_test_footer(tc_name)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

# Copyright (c) 2021 by
# Donatas Abraitis <donatas.abraitis@gmail.com>
#

"""
Test if Opaque Data is accessable from other daemons in Zebra
"""

import os
import sys
import json
import pytest
import functools

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen

pytestmark = [pytest.mark.bgpd, pytest.mark.ospfd, pytest.mark.ospf6d]


def setup_module(mod):
    topodef = {"s1": ("r1", "r2"), "s2": ("r3", "r4")}
    tgen = Topogen(topodef, mod.__name__)
    tgen.start_topology()

    router_list = tgen.routers()

    for _, (rname, router) in enumerate(router_list.items(), 1):
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_BGP, os.path.join(CWD, "{}/bgpd.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_OSPF, os.path.join(CWD, "{}/ospfd.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_OSPF6, os.path.join(CWD, "{}/ospf6d.conf".format(rname))
        )

    tgen.start_router()


def teardown_module():
    tgen = get_topogen()
    tgen.stop_topology()


def test_zebra_opaque():
    tgen = get_topogen()

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    def _bgp_converge(router):
        output = json.loads(router.vtysh_cmd("show ip route 192.168.1.0/24 json"))
        expected = {
            "192.168.1.0/24": [
                {
                    "communities": "65002:1 65002:2",
                    "largeCommunities": "65002:1:1 65002:2:1",
                }
            ]
        }
        return topotest.json_cmp(output, expected)

    def _ospf_converge(router):
        output = json.loads(router.vtysh_cmd("show ip route 192.168.1.0/24 json"))
        expected = {
            "192.168.1.0/24": [
                {
                    "ospfPathType": "Intra-Area",
                    "ospfAreaId": "0.0.0.0",
                }
            ]
        }
        return topotest.json_cmp(output, expected)

    def _ospf6_converge(router):
        output = json.loads(router.vtysh_cmd("show ipv6 route 2001:db8:1::/64 json"))
        expected = {
            "2001:db8:1::/64": [
                {
                    "ospfPathType": "Intra-Area",
                    "ospfAreaId": "0.0.0.0",
                }
            ]
        }
        return topotest.json_cmp(output, expected)

    router = tgen.gears["r1"]
    test_func = functools.partial(_bgp_converge, router)
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=0.5)
    assert result is None, 'Cannot see BGP community aliases "{}"'.format(router)

    router = tgen.gears["r3"]
    test_func = functools.partial(_ospf_converge, router)
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=0.5)
    assert result is None, 'Cannot see OSPFv2 opaque attributes "{}"'.format(router)

    router = tgen.gears["r3"]
    test_func = functools.partial(_ospf6_converge, router)
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=0.5)
    assert result is None, 'Cannot see OSPFv3 opaque attributes "{}"'.format(router)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



from lib.lutil import luCommand

luCommand(
    "r1", 'vtysh -c "show bgp nexthop"', "99.0.0.. valid", "wait", "See CE static NH"
)
luCommand(
    "r3", 'vtysh -c "show bgp nexthop"', "99.0.0.. valid", "wait", "See CE static NH"
)
luCommand(
    "r4", 'vtysh -c "show bgp nexthop"', "99.0.0.. valid", "wait", "See CE static NH"
)
luCommand("r1", 'vtysh -c "show bgp ipv4 uni"', "i 5.*i 5", "wait", "See CE routes")
luCommand("r3", 'vtysh -c "show bgp ipv4 uni"', "i 5.*i 5", "wait", "See CE routes")
luCommand("r4", 'vtysh -c "show bgp ipv4 uni"', "i 5.*i 5", "wait", "See CE routes")
luCommand("ce1", 'vtysh -c "show bgp ipv4 uni 5.1.0.0/24"', "", "none", "See CE routes")
luCommand("r1", 'vtysh -c "show bgp ipv4 uni 5.1.0.0/24"', "", "none", "See CE routes")
luCommand("ce2", 'vtysh -c "show bgp ipv4 uni 5.1.0.0/24"', "", "none", "See CE routes")
luCommand("r3", 'vtysh -c "show bgp ipv4 uni 5.1.0.0/24"', "", "none", "See CE routes")
luCommand("ce3", 'vtysh -c "show bgp ipv4 uni 5.1.2.0/24"', "", "none", "See CE routes")
luCommand("r4", 'vtysh -c "show bgp ipv4 uni 5.1.2.0/24"', "", "none", "See CE routes")

luCommand(
    "r1", 'vtysh -c "add vrf cust1 prefix 99.0.0.1/32"', ".", "none", "IP Address"
)
luCommand(
    "r1",
    'vtysh -c "show vnc registrations local"',
    "99.0.0.1",
    "wait",
    "Local Registration",
)
luCommand(
    "r1",
    'vtysh -c "show vnc registrations imported"',
    "2 out of 2 imported",
    "wait",
    "Imported Registrations",
)
luCommand(
    "r3",
    'vtysh -c "show bgp ipv4 vpn"',
    "i 99.0.0.1/32",
    "wait",
    "See R1s static address",
)
luCommand(
    "r4",
    'vtysh -c "show bgp ipv4 vpn"',
    "i 99.0.0.1/32",
    "wait",
    "See R1s static address",
)
luCommand(
    "r3", 'vtysh -c "show bgp ipv4 vpn rd 10:1"', "i 5.*i 5", "wait", "See R1s imports"
)
luCommand(
    "r4", 'vtysh -c "show bgp ipv4 vpn rd 10:1"', "i 5.*i 5", "wait", "See R1s imports"
)

luCommand(
    "r3", 'vtysh -c "add vrf cust1 prefix 99.0.0.2/32"', ".", "none", "IP Address"
)
luCommand(
    "r3",
    'vtysh -c "show vnc registrations local"',
    "99.0.0.2",
    "wait",
    "Local Registration",
)
have2ndImports = luCommand(
    "r3",
    'vtysh -c "show vnc registrations imported"',
    "2 out of 2 imported",
    "none",
    "Imported Registrations",
    2,
)
if have2ndImports:
    luCommand(
        "r3",
        'vtysh -c "show vnc registrations imported"',
        "2 out of 2 imported",
        "pass",
        "Imported Registrations",
    )
luCommand(
    "r1",
    'vtysh -c "show bgp ipv4 vpn"',
    "i 99.0.0.2/32",
    "wait",
    "See R3s static address",
)
luCommand(
    "r4",
    'vtysh -c "show bgp ipv4 vpn"',
    "i 99.0.0.2/32",
    "wait",
    "See R3s static address",
)
if have2ndImports:
    luCommand(
        "r1",
        'vtysh -c "show bgp ipv4 vpn rd 10:3"',
        "i 5.*i 5",
        "none",
        "See R3s imports",
    )
    luCommand(
        "r4",
        'vtysh -c "show bgp ipv4 vpn rd 10:3"',
        "i 5.*i 5",
        "none",
        "See R3s imports",
    )

luCommand(
    "r4", 'vtysh -c "add vrf cust1 prefix 99.0.0.3/32"', ".", "none", "IP Address"
)
luCommand(
    "r4",
    'vtysh -c "show vnc registrations local"',
    "99.0.0.3",
    "wait",
    "Local Registration",
)
luCommand(
    "r4",
    'vtysh -c "show vnc registrations imported"',
    "2 out of 2 imported",
    "wait",
    "Imported Registrations",
)
luCommand(
    "r1",
    'vtysh -c "show bgp ipv4 vpn"',
    "i 99.0.0.3/32",
    "wait",
    "See R4s static address",
)
luCommand(
    "r3",
    'vtysh -c "show bgp ipv4 vpn"',
    "i 99.0.0.3/32",
    "wait",
    "See R4s static address",
)
luCommand(
    "r1", 'vtysh -c "show bgp ipv4 vpn rd 10:4"', "i 5.*i 5", "wait", "See R4s imports"
)
luCommand(
    "r3", 'vtysh -c "show bgp ipv4 vpn rd 10:4"', "i 5.*i 5", "wait", "See R4s imports"
)


luCommand(
    "r1",
    'vtysh -c "show vnc registrations remote"',
    "5.1.2.0/24 .*5.1.3.0/24",
    "wait",
    "R4s registrations",
)
luCommand(
    "r3",
    'vtysh -c "show vnc registrations remote"',
    "5.1.2.0/24 .*5.1.3.0/24",
    "wait",
    "R4s registrations",
)
if have2ndImports:
    luCommand(
        "r1",
        'vtysh -c "show vnc registrations remote"',
        "5.1.0.0/24 .*5.1.1.0/24",
        "wait",
        "Remote registrations",
    )
    luCommand(
        "r3",
        'vtysh -c "show vnc registrations remote"',
        "5.1.0.0/24 .*5.1.1.0/24",
        "wait",
        "Remote registrations",
    )
luCommand(
    "r4",
    'vtysh -c "show vnc registrations remote"',
    "5.1.0.0/24 .*5.1.1.0/24",
    "wait",
    "Remote registrations",
)
luCommand("r1", 'vtysh -c "show vnc registrations"', ".", "none")
luCommand("r3", 'vtysh -c "show vnc registrations"', ".", "none")
luCommand("r4", 'vtysh -c "show vnc registrations"', ".", "none")



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_bgp_evpn.py
# Part of NetDEF Topology Tests
#
# Copyright (c) 2019 by 6WIND
#

"""
 test_bgp_evpn.py: Test the FRR BGP daemon with BGP IPv6 interface
 with route advertisements on a separate netns.
"""

import json
from functools import partial
import os
import sys
import pytest
import platform

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.bgp import verify_bgp_rib
from lib.common_config import apply_raw_config
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger

# Required to instantiate the topology builder class.

pytestmark = [pytest.mark.bgpd]


def build_topo(tgen):
    "Build function"

    tgen.add_router("r1")
    tgen.add_router("r2")
    tgen.add_router("r3")

    switch = tgen.add_switch("s1")
    switch.add_link(tgen.gears["r1"])
    switch.add_link(tgen.gears["r2"])
    switch.add_link(tgen.gears["r3"])

    switch = tgen.add_switch("s2")
    switch.add_link(tgen.gears["r1"])

    switch = tgen.add_switch("s3")
    switch.add_link(tgen.gears["r2"])

    switch = tgen.add_switch("s4")
    switch.add_link(tgen.gears["r3"])


def setup_module(mod):
    "Sets up the pytest environment"

    tgen = Topogen(build_topo, mod.__name__)
    tgen.start_topology()

    router_list = tgen.routers()

    krel = platform.release()
    if topotest.version_cmp(krel, "4.18") < 0:
        logger.info(
            'BGP EVPN RT5 NETNS tests will not run (have kernel "{}", but it requires 4.18)'.format(
                krel
            )
        )
        return pytest.skip("Skipping BGP EVPN RT5 NETNS Test. Kernel not supported")

    # create VRF vrf-101 on R1, R2, R3
    # create loop101
    cmds_vrflite = [
        "ip link add {0}-vrf-{1} type vrf table {1}",
        "ip ru add oif {0}-vrf-{1} table {1}",
        "ip ru add iif {0}-vrf-{1} table {1}",
        "ip link set dev {0}-vrf-{1} up",
        "ip link add loop{1} type dummy",
        "ip link set dev loop{1} master {0}-vrf-{1}",
        "ip link set dev loop{1} up",
    ]

    cmds_r2 = [  # config routing 101
        "ip link add name bridge-101 up type bridge stp_state 0",
        "ip link set bridge-101 master {}-vrf-101",
        "ip link set dev bridge-101 up",
        "ip link add name vxlan-101 type vxlan id 101 dstport 4789 dev r2-eth0 local 192.168.100.41",
        "ip link set dev vxlan-101 master bridge-101",
        "ip link set vxlan-101 up type bridge_slave learning off flood off mcast_flood off",
    ]

    cmds_r3 = [  # config routing 102
        "ip link add name bridge-102 up type bridge stp_state 0",
        "ip link set bridge-102 master {}-vrf-102",
        "ip link set dev bridge-102 up",
        "ip link add name vxlan-102 type vxlan id 102 dstport 4789 dev r3-eth0 local 192.168.100.61",
        "ip link set dev vxlan-102 master bridge-102",
        "ip link set vxlan-102 up type bridge_slave learning off flood off mcast_flood off",
    ]

    # cmds_r1_netns_method3 = [
    #     "ip link add name vxlan-{1} type vxlan id {1} dstport 4789 dev {0}-eth0 local 192.168.100.21",
    #     "ip link set dev vxlan-{1} netns {0}-vrf-{1}",
    #     "ip netns exec {0}-vrf-{1} ip li set dev lo up",
    #     "ip netns exec {0}-vrf-{1} ip link add name bridge-{1} up type bridge stp_state 0",
    #     "ip netns exec {0}-vrf-{1} ip link set dev vxlan-{1} master bridge-{1}",
    #     "ip netns exec {0}-vrf-{1} ip link set bridge-{1} up",
    #     "ip netns exec {0}-vrf-{1} ip link set vxlan-{1} up",
    # ]

    router = tgen.gears["r1"]

    ns = "r1-vrf-101"
    tgen.net["r1"].add_netns(ns)
    tgen.net["r1"].cmd_raises("ip link add loop101 type dummy")
    tgen.net["r1"].set_intf_netns("loop101", ns, up=True)

    router = tgen.gears["r2"]
    for cmd in cmds_vrflite:
        logger.info("cmd to r2: " + cmd.format("r2", 101))
        output = router.cmd_raises(cmd.format("r2", 101))
        logger.info("result: " + output)

    for cmd in cmds_r2:
        logger.info("cmd to r2: " + cmd.format("r2"))
        output = router.cmd_raises(cmd.format("r2"))
        logger.info("result: " + output)

    router = tgen.gears["r3"]
    for cmd in cmds_vrflite:
        logger.info("cmd to r3: " + cmd.format("r3", 102))
        output = router.cmd_raises(cmd.format("r3", 102))
        logger.info("result: " + output)

    for cmd in cmds_r3:
        logger.info("cmd to r3: " + cmd.format("r3"))
        output = router.cmd_raises(cmd.format("r3"))
        logger.info("result: " + output)

    tgen.net["r1"].cmd_raises(
        "ip link add name vxlan-101 type vxlan id 101 dstport 4789 dev r1-eth0 local 192.168.100.21"
    )
    tgen.net["r1"].set_intf_netns("vxlan-101", "r1-vrf-101", up=True)
    tgen.net["r1"].cmd_raises("ip -n r1-vrf-101 link set lo up")
    tgen.net["r1"].cmd_raises(
        "ip -n r1-vrf-101 link add name bridge-101 up type bridge stp_state 0"
    )
    tgen.net["r1"].cmd_raises(
        "ip -n r1-vrf-101 link set dev vxlan-101 master bridge-101"
    )
    tgen.net["r1"].cmd_raises("ip -n r1-vrf-101 link set bridge-101 up")
    tgen.net["r1"].cmd_raises("ip -n r1-vrf-101 link set vxlan-101 up")

    for rname, router in tgen.routers().items():
        logger.info("Loading router %s" % rname)
        if rname == "r1":
            router.use_netns_vrf()
            router.load_frr_config(os.path.join(CWD, "{}/frr.conf".format(rname)))
        else:
            router.load_frr_config(os.path.join(CWD, "{}/frr.conf".format(rname)))

    # Initialize all routers.
    tgen.start_router()


def teardown_module(_mod):
    "Teardown the pytest environment"
    tgen = get_topogen()

    tgen.net["r1"].delete_netns("r1-vrf-101")
    tgen.stop_topology()


def _test_evpn_ping_router(pingrouter, ipv4_only=False):
    """
    internal function to check ping between r1 and r2
    """
    # Check IPv4 and IPv6 connectivity between r1 and r2 ( routing vxlan evpn)
    logger.info(
        "Check Ping IPv4 from  R1(r1-vrf-101) to R2(r2-vrf-101 = 192.168.101.41)"
    )
    output = pingrouter.run("ip netns exec r1-vrf-101 ping 192.168.101.41 -f -c 1000")
    logger.info(output)
    if "1000 packets transmitted, 1000 received" not in output:
        assertmsg = (
            "expected ping IPv4 from R1(r1-vrf-101) to R2(192.168.101.41) should be ok"
        )
        assert 0, assertmsg
    else:
        logger.info("Check Ping IPv4 from R1(r1-vrf-101) to R2(192.168.101.41) OK")

    if ipv4_only:
        return

    logger.info("Check Ping IPv6 from  R1(r1-vrf-101) to R2(r2-vrf-101 = fd00::2)")
    output = pingrouter.run("ip netns exec r1-vrf-101 ping fd00::2 -f -c 1000")
    logger.info(output)
    if "1000 packets transmitted, 1000 received" not in output:
        assert 0, "expected ping IPv6 from R1(r1-vrf-101) to R2(fd00::2) should be ok"
    else:
        logger.info("Check Ping IPv6 from R1(r1-vrf-101) to R2(fd00::2) OK")


def test_protocols_convergence():
    """
    Assert that all protocols have converged
    statuses as they depend on it.
    """
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)
    # Check BGP IPv4 routing tables on r1
    logger.info("Checking BGP L2VPN EVPN routes for convergence on r1")

    for rname in ("r1", "r2"):
        router = tgen.gears[rname]
        json_file = "{}/{}/bgp_l2vpn_evpn_routes.json".format(CWD, router.name)
        if not os.path.isfile(json_file):
            assert 0, "bgp_l2vpn_evpn_routes.json file not found"

        expected = json.loads(open(json_file).read())
        test_func = partial(
            topotest.router_json_cmp,
            router,
            "show bgp l2vpn evpn json",
            expected,
        )
        _, result = topotest.run_and_expect(test_func, None, count=20, wait=1)
        assertmsg = '"{}" JSON output mismatches'.format(router.name)
        assert result is None, assertmsg


def test_protocols_dump_info():
    """
    Dump EVPN information
    """
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)
    # Check IPv4/IPv6 routing tables.
    output = tgen.gears["r1"].vtysh_cmd("show bgp l2vpn evpn", isjson=False)
    logger.info("==== result from show bgp l2vpn evpn")
    logger.info(output)
    output = tgen.gears["r1"].vtysh_cmd(
        "show bgp l2vpn evpn route detail", isjson=False
    )
    logger.info("==== result from show bgp l2vpn evpn route detail")
    logger.info(output)
    output = tgen.gears["r1"].vtysh_cmd("show bgp vrf r1-vrf-101 ipv4", isjson=False)
    logger.info("==== result from show bgp vrf r1-vrf-101 ipv4")
    logger.info(output)
    output = tgen.gears["r1"].vtysh_cmd("show bgp vrf r1-vrf-101 ipv6", isjson=False)
    logger.info("==== result from show bgp vrf r1-vrf-101 ipv6")
    logger.info(output)
    output = tgen.gears["r1"].vtysh_cmd("show bgp vrf r1-vrf-101", isjson=False)
    logger.info("==== result from show bgp vrf r1-vrf-101 ")
    logger.info(output)
    output = tgen.gears["r1"].vtysh_cmd("show ip route vrf r1-vrf-101", isjson=False)
    logger.info("==== result from show ip route vrf r1-vrf-101")
    logger.info(output)
    output = tgen.gears["r1"].vtysh_cmd("show ipv6 route vrf r1-vrf-101", isjson=False)
    logger.info("==== result from show ipv6 route vrf r1-vrf-101")
    logger.info(output)
    output = tgen.gears["r1"].vtysh_cmd("show evpn vni detail", isjson=False)
    logger.info("==== result from show evpn vni detail")
    logger.info(output)
    output = tgen.gears["r1"].vtysh_cmd("show evpn next-hops vni all", isjson=False)
    logger.info("==== result from show evpn next-hops vni all")
    logger.info(output)
    output = tgen.gears["r1"].vtysh_cmd("show evpn rmac vni all", isjson=False)
    logger.info("==== result from show evpn rmac vni all")
    logger.info(output)


def test_router_check_ip():
    """
    Check routes are correctly installed
    """
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    expected = {
        "fd00::2/128": [
            {
                "prefix": "fd00::2/128",
                "vrfName": "r1-vrf-101",
                "nexthops": [
                    {
                        "ip": "::ffff:192.168.100.41",
                    }
                ],
            }
        ]
    }
    result = topotest.router_json_cmp(
        tgen.gears["r1"], "show ipv6 route vrf r1-vrf-101 fd00::2/128 json", expected
    )
    assert result is None, "ipv6 route check failed"


def _test_router_check_evpn_contexts(router, ipv4_only=False):
    """
    Check EVPN nexthops and RMAC number  are correctly configured
    """
    if ipv4_only:
        expected = {
            "101": {
                "numNextHops": 1,
                "192.168.100.41": {
                    "nexthopIp": "192.168.100.41",
                },
            }
        }
    else:
        expected = {
            "101": {
                "numNextHops": 2,
                "192.168.100.41": {
                    "nexthopIp": "192.168.100.41",
                },
                "::ffff:192.168.100.41": {
                    "nexthopIp": "::ffff:192.168.100.41",
                },
            }
        }
    result = topotest.router_json_cmp(
        router, "show evpn next-hops vni all json", expected
    )
    assert result is None, "evpn next-hops check failed"

    expected = {"101": {"numRmacs": 1}}
    result = topotest.router_json_cmp(router, "show evpn rmac vni all json", expected)
    assert result is None, "evpn rmac number check failed"


def test_router_check_evpn_contexts():
    """
    Check EVPN nexthops and RMAC number  are correctly configured
    """
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    _test_router_check_evpn_contexts(tgen.gears["r1"])


def test_evpn_ping():
    """
    Check ping between R1 and R2 is ok
    """
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    _test_evpn_ping_router(tgen.gears["r1"])


def test_evpn_disable_routemap():
    """
    Check the removal of a route-map on R2. More EVPN Prefixes are expected
    """
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    tgen.gears["r2"].vtysh_cmd(
        """
        configure terminal\n
        router bgp 65000 vrf r2-vrf-101\n
        address-family l2vpn evpn\n
        advertise ipv4 unicast\n
        advertise ipv6 unicast\n
        """
    )
    router = tgen.gears["r1"]
    json_file = "{}/{}/bgp_l2vpn_evpn_routes_all.json".format(CWD, router.name)
    if not os.path.isfile(json_file):
        assert 0, "bgp_l2vpn_evpn_routes.json file not found"

    expected = json.loads(open(json_file).read())
    test_func = partial(
        topotest.router_json_cmp,
        router,
        "show bgp l2vpn evpn json",
        expected,
    )
    _, result = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assertmsg = '"{}" JSON output mismatches'.format(router.name)
    assert result is None, assertmsg


def test_evpn_remove_ip():
    """
    Check the removal of an EVPN route is correctly handled
    """
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    config_no_ipv6 = {
        "r2": {
            "raw_config": [
                "router bgp 65000 vrf r2-vrf-101",
                "address-family ipv6 unicast",
                "no network fd00::3/128",
                "no network fd00::2/128",
            ]
        }
    }

    logger.info("==== Remove IPv6 network on R2")
    result = apply_raw_config(tgen, config_no_ipv6)
    assert result is True, "Failed to remove IPv6 network on R2, Error: {} ".format(
        result
    )
    ipv6_routes = {
        "r1": {
            "static_routes": [
                {
                    "vrf": "r1-vrf-101",
                    "network": ["fd00::2/128"],
                }
            ]
        }
    }
    result = verify_bgp_rib(tgen, "ipv6", "r1", ipv6_routes, expected=False)
    assert result is not True, "expect IPv6 route fd00::2/128 withdrawn"

    output = tgen.gears["r1"].vtysh_cmd("show evpn next-hops vni all", isjson=False)
    logger.info("==== result from show evpn next-hops vni all")
    logger.info(output)
    output = tgen.gears["r1"].vtysh_cmd("show evpn rmac vni all", isjson=False)
    logger.info("==== result from show evpn next-hops vni all")
    logger.info(output)


def test_router_check_evpn_contexts_again():
    """
    Check EVPN nexthops and RMAC number  are correctly configured
    """
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    _test_router_check_evpn_contexts(tgen.gears["r1"], ipv4_only=True)


def test_evpn_ping_again():
    """
    Check ping between R1 and R2 is ok
    """
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    _test_evpn_ping_router(tgen.gears["r1"], ipv4_only=True)


def test_memory_leak():
    "Run the memory leak test and report results."
    tgen = get_topogen()
    if not tgen.is_memleak_enabled():
        pytest.skip("Memory leak test/report is disabled")

    tgen.report_memory_leaks()


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_pim_mrib.py
#
# Copyright (c) 2024 ATCorp
# Nathan Bahr
#

import os
import sys
import pytest
from functools import partial

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, get_topogen
from lib.topolog import logger
from lib.pim import (
    verify_pim_rp_info,
    verify_upstream_iif,
    McastTesterHelper,
)
from lib.common_config import step, write_test_header

"""
test_pim_mrib.py: Test PIM MRIB overrides and RPF modes
"""

TOPOLOGY = """
   Test PIM MRIB overrides and RPF modes
     Static routes installed that uses R2 to get between R1 and R4.
     Tests will install MRIB override through R3

            +---+---+                      +---+---+
            |       |     10.0.0.0/24      |       |
            +  R1   +----------------------+  R2   |
            |       | .1                .2 |       |
            +---+---+ r1-eth0      r2-eth0 +---+---+
             .1 | r1-eth1              r2-eth1 | .2
                |                              |
   10.0.1.0/24  |                              |  10.0.2.0/24
                |                              |
             .3 | r3-eth0              r4-eth0 | .4
            +---+---+ r3-eth1      r4-eth1 +---+---+
            |       | .3                .4 |       |
            +  R3   +----------------------+  R4   |---r4-dum0 10.10.0.4/24
            |       |      10.0.3.0/24     |       |
            +---+---+                      +---+---+
"""

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# Required to instantiate the topology builder class.
pytestmark = [pytest.mark.pimd]

GROUP1 = "239.1.1.1"
GROUP2 = "239.2.2.2"


def build_topo(tgen):
    """Build function"""

    # Create routers
    tgen.add_router("r1")
    tgen.add_router("r2")
    tgen.add_router("r3")
    tgen.add_router("r4")

    # Create topology links
    tgen.add_link(tgen.gears["r1"], tgen.gears["r2"], "r1-eth0", "r2-eth0")
    tgen.add_link(tgen.gears["r1"], tgen.gears["r3"], "r1-eth1", "r3-eth0")
    tgen.add_link(tgen.gears["r2"], tgen.gears["r4"], "r2-eth1", "r4-eth0")
    tgen.add_link(tgen.gears["r3"], tgen.gears["r4"], "r3-eth1", "r4-eth1")

    tgen.gears["r4"].run("ip link add r4-dum0 type dummy")


def setup_module(mod):
    logger.info("PIM MRIB/RPF functionality:\n {}".format(TOPOLOGY))

    tgen = Topogen(build_topo, mod.__name__)
    tgen.start_topology()

    router_list = tgen.routers()
    for rname, router in router_list.items():
        logger.info("Loading router %s" % rname)
        router.load_frr_config(os.path.join(CWD, "{}/frr.conf".format(rname)))

    # Initialize all routers.
    tgen.start_router()


def teardown_module(mod):
    """Teardown the pytest environment"""
    tgen = get_topogen()
    tgen.stop_topology()


def test_pim_mrib_init(request):
    """Test boot in MRIB-than-URIB with the default MRIB"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    step("Verify rp-info using default URIB nexthop")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth0",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth0",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_override(request):
    """Test MRIB override nexthop"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Install a MRIB route that has a shorter prefix length and lower cost.
    # In MRIB-than-URIB mode, it should use this route
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         ip mroute 10.0.0.0/16 10.0.3.3 25
        exit
        """
    )

    step("Verify rp-info using MRIB nexthop")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth1",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth1",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_prefix_mode(request):
    """Test longer prefix lookup mode"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Switch to longer prefix match, should switch back to the URIB route
    # even with the lower cost, the longer prefix match will win because of the mode
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         router pim
          rpf-lookup-mode longer-prefix
         exit
        exit
        """
    )

    step("Verify rp-info using URIB nexthop")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth0",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth0",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_dist_mode(request):
    """Test lower distance lookup mode"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Switch to lower distance match, should switch back to the MRIB route
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         router pim
          rpf-lookup-mode lower-distance
         exit
        exit
        """
    )

    step("Verify rp-info using MRIB nexthop")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth1",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth1",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_urib_mode(request):
    """Test URIB only lookup mode"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Switch to urib only match, should switch back to the URIB route
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         router pim
          rpf-lookup-mode urib-only
         exit
        exit
        """
    )

    step("Verify rp-info using URIB nexthop")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth0",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth0",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_mrib_mode(request):
    """Test MRIB only lookup mode"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Switch to mrib only match, should switch back to the MRIB route
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         router pim
          rpf-lookup-mode mrib-only
         exit
        exit
        """
    )

    step("Verify rp-info using MRIB nexthop")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth1",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth1",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_mrib_mode_no_route(request):
    """Test MRIB only with no route"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Remove the MRIB route, in mrib-only mode, it should switch to no path for the RP
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         no ip mroute 10.0.0.0/16 10.0.3.3 25
        exit
        """
    )

    step("Verify rp-info with Unknown next hop")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "Unknown",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "Unknown",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_rpf_lookup_source_list_init(request):
    """Test RPF lookup source list with initial setup"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Reset back to mrib then urib mode
    # Also add mode using SRCPLIST(10.0.0.1) and SRCPLIST2(10.0.1.1)
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         router pim
          rpf-lookup-mode mrib-then-urib
          rpf-lookup-mode mrib-then-urib source-list SRCPLIST
          rpf-lookup-mode mrib-then-urib source-list SRCPLIST2
         exit
        exit
        """
    )

    step("Verify rp-info with default next hop")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth0",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth0",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_rpf_lookup_source_list_add_mroute(request):
    """Test RPF lookup source list with MRIB route on alternate path"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Add a MRIB route through r4-eth1 that is better distance but worse prefix
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         ip mroute 10.0.0.0/16 10.0.3.3 25
        exit
        """
    )

    step("Verify rp-info with MRIB next hop")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth1",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth1",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_rpf_lookup_source_list_src1_prefix_mode(request):
    """Test RPF lookup source list src1 longer prefix mode"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Switch just source 1 to longest prefix
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         router pim
          rpf-lookup-mode longer-prefix source-list SRCPLIST
         exit
        exit
        """
    )

    step("Verify rp-info with URIB next hop for source 1 and MRIB for source 2")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth0",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth1",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_rpf_lookup_source_list_src1_dist_src2_prefix_mode(request):
    """Test RPF lookup source list src1 lower distance mode and src2 longer prefix mode"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Switch source 1 to shortest distance, source 2 to longest prefix
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         router pim
          rpf-lookup-mode lower-distance source-list SRCPLIST
          rpf-lookup-mode longer-prefix source-list SRCPLIST2
         exit
        exit
        """
    )

    step("Verify rp-info with MRIB next hop for source 1 and URIB for source 2")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth1",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth0",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_rpf_lookup_source_list_src1_urib_src2_dist_mode(request):
    """Test RPF lookup source list src1 urib mode and src2 lower distance mode"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Switch source 1 to urib only, source 2 to shorter distance
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         router pim
          rpf-lookup-mode urib-only source-list SRCPLIST
          rpf-lookup-mode lower-distance source-list SRCPLIST2
         exit
        exit
        """
    )

    step("Verify rp-info with URIB next hop for source 1 and MRIB for source 2")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth0",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth1",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_rpf_lookup_source_list_src1_mrib_src2_urib_mode(request):
    """Test RPF lookup source list src1 mrib mode and src2 urib mode"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Switch source 1 to mrib only, source 2 to urib only
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         router pim
          rpf-lookup-mode mrib-only source-list SRCPLIST
          rpf-lookup-mode urib-only source-list SRCPLIST2
         exit
        exit
        """
    )

    step("Verify rp-info with MRIB next hop for source 1 and URIB for source 2")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth1",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth0",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_rpf_lookup_source_list_removed(request):
    """Test RPF lookup source list removed"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Remove both special modes, both should switch to MRIB route
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         router pim
          no rpf-lookup-mode mrib-only source-list SRCPLIST
          no rpf-lookup-mode urib-only source-list SRCPLIST2
         exit
        exit
        """
    )

    step("Verify rp-info with MRIB next hop for both sources")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth1",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth1",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_rpf_lookup_source_list_del_mroute(request):
    """Test RPF lookup source list delete mroute"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    # Remove the MRIB route, both should switch to URIB
    tgen.routers()["r4"].vtysh_cmd(
        """
        conf term
         no ip mroute 10.0.0.0/16 10.0.3.3 25
        exit
        """
    )

    step("Verify rp-info with URIB next hop for both sources")
    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "224.0.0.0/4",
        "r4-eth0",
        "10.0.0.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_pim_rp_info(
        tgen,
        None,
        "r4",
        "225.0.0.0/24",
        "r4-eth0",
        "10.0.1.1",
        "Static",
        False,
        "ipv4",
        True,
    )
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)


def test_pim_mrib_rpf_lookup_group_list(request):
    """Test RPF lookup group list"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    with McastTesterHelper(tgen) as apphelper:
        step(
            ("Send multicast traffic from R1 to dense groups {}, {}").format(
                GROUP1, GROUP2
            )
        )
        result = apphelper.run_traffic("r1", [GROUP1, GROUP2], bind_intf="r1-eth1")
        assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

        # Reset back to mrib then urib mode
        # Also add mode using GRPPLIST(239.1.1.1) and GRPPLIST2(239.2.2.2)
        # And do an igmp join to both groups on r4-eth2
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             router pim
              rpf-lookup-mode mrib-then-urib
              rpf-lookup-mode mrib-then-urib group-list GRPPLIST
              rpf-lookup-mode mrib-then-urib group-list GRPPLIST2
             exit
             int r4-dum0
              ip igmp join-group {}
              ip igmp join-group {}
             exit
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif with default next hop")
        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step("Add MRIB route through alternate path")
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             ip mroute 10.0.0.0/16 10.0.3.3 25
            exit
            """
        )

        step("Verify upstream iif with alternate next hop")
        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step("Switch group1 to longer prefix match (URIB)")
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             router pim
              rpf-lookup-mode longer-prefix group-list GRPPLIST
             exit
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif of group1 is URIB, group2 is MRIB")
        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step(
            "Switch group1 to lower distance match (MRIB), and group2 to longer prefix (URIB)"
        )
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             router pim
              rpf-lookup-mode lower-distance group-list GRPPLIST
              rpf-lookup-mode longer-prefix group-list GRPPLIST2
             exit
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif of group1 is MRIB, group2 is URIB")
        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step("Switch group1 to urib match only, and group2 to lower distance (URIB)")
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             router pim
              rpf-lookup-mode urib-only group-list GRPPLIST
              rpf-lookup-mode lower-distance group-list GRPPLIST2
             exit
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif of group1 is URIB, group2 is MRIB")
        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step("Switch group1 to mrib match only, and group2 to urib match only")
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             router pim
              rpf-lookup-mode mrib-only group-list GRPPLIST
              rpf-lookup-mode urib-only group-list GRPPLIST2
             exit
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif of group1 is MRIB, group2 is URIB")
        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step("Delete MRIB route")
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             no ip mroute 10.0.0.0/16 10.0.3.3 25
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif of group1 is Unknown, group2 is URIB")
        result = verify_upstream_iif(
            tgen, "r4", "Unknown", "10.0.1.1", GROUP1, "NotJoined"
        )
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )


def test_pim_mrib_rpf_lookup_source_group_lists(request):
    """Test RPF lookup source and group lists"""
    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)

    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    with McastTesterHelper(tgen) as apphelper:
        step(
            ("Send multicast traffic from R1 to dense groups {}, {}").format(
                GROUP1, GROUP2
            )
        )
        result = apphelper.run_traffic("r1", [GROUP1, GROUP2], bind_intf="r1-eth1")
        assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

        # Reset back to mrib then urib mode
        # Also add mode using GRPPLIST(239.1.1.1) and GRPPLIST2(239.2.2.2), both using SRCPLIST2
        # And do an igmp join to both groups on r4-eth2
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             router pim
              rpf-lookup-mode mrib-then-urib
              rpf-lookup-mode mrib-then-urib group-list GRPPLIST source-list SRCPLIST2
              rpf-lookup-mode mrib-then-urib group-list GRPPLIST2 source-list SRCPLIST2
             exit
             int r4-dum0
              ip igmp join-group {}
              ip igmp join-group {}
             exit
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif with default next hop")
        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step("Add MRIB route through alternate path")
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             ip mroute 10.0.0.0/16 10.0.3.3 25
            exit
            """
        )

        step("Verify upstream iif with alternate next hop")
        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step("Switch group1 to longer prefix match (URIB)")
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             router pim
              rpf-lookup-mode longer-prefix group-list GRPPLIST source-list SRCPLIST2
             exit
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif of group1 is URIB, group2 is MRIB")
        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step(
            "Switch group1 to lower distance match (MRIB), and group2 to longer prefix (URIB)"
        )
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             router pim
              rpf-lookup-mode lower-distance group-list GRPPLIST source-list SRCPLIST2
              rpf-lookup-mode longer-prefix group-list GRPPLIST2 source-list SRCPLIST2
             exit
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif of group1 is MRIB, group2 is URIB")
        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step("Switch group1 to urib match only, and group2 to lower distance (URIB)")
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             router pim
              rpf-lookup-mode urib-only group-list GRPPLIST source-list SRCPLIST2
              rpf-lookup-mode lower-distance group-list GRPPLIST2 source-list SRCPLIST2
             exit
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif of group1 is URIB, group2 is MRIB")
        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step("Switch group1 to mrib match only, and group2 to urib match only")
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             router pim
              rpf-lookup-mode mrib-only group-list GRPPLIST source-list SRCPLIST2
              rpf-lookup-mode urib-only group-list GRPPLIST2 source-list SRCPLIST2
             exit
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif of group1 is MRIB, group2 is URIB")
        result = verify_upstream_iif(tgen, "r4", "r4-eth1", "10.0.1.1", GROUP1)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        step("Delete MRIB route")
        tgen.routers()["r4"].vtysh_cmd(
            """
            conf term
             no ip mroute 10.0.0.0/16 10.0.3.3 25
            exit
            """.format(
                GROUP1, GROUP2
            )
        )

        step("Verify upstream iif of group1 is Unknown, group2 is URIB")
        result = verify_upstream_iif(
            tgen, "r4", "Unknown", "10.0.1.1", GROUP1, "NotJoined"
        )
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_upstream_iif(tgen, "r4", "r4-eth0", "10.0.1.1", GROUP2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )


def test_memory_leak():
    """Run the memory leak test and report results."""
    tgen = get_topogen()
    if not tgen.is_memleak_enabled():
        pytest.skip("Memory leak test/report is disabled")

    tgen.report_memory_leaks()


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# Part of NetDEF Topology Tests
#
# Copyright 2021 by LINE Corporation, Hiroki Shirokura <hiroki.shirokura@linecorp.com>
# Copyright 2023 6WIND S.A.

"""
test_isis_sr_flex_algo_topo1.py:

[+] Flex-Algos 201 exclude red
[+] Flex-Algos 202 exclude blue
[+] Flex-Algos 203 exclude green
[+] Flex-Algos 204 include-any blue green
[+] Flex-Algos 205 include-any red green
[+] Flex-Algos 206 include-any red blue
[+] Flex-Algos 207 include-all yellow orange

     +--------+  10.12.0.0/24  +--------+
     |        |       red      |        |
     |   RT1  |----------------|   RT2  |
     |        |                |        |
     +--------+                +--------+
  10.13.0.0/24 \\             / 10.23.0.0/24
          green \\           / blue
         yellow  \\         / yellow
          orange  +--------+ orange
                  |        |
                  |   RT3  |
                  |        |
                  +--------+
"""

import os
import sys
import pytest
import json
from copy import deepcopy
from functools import partial

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger


pytestmark = [pytest.mark.isisd]

# Global multi-dimensional dictionary containing all expected outputs
outputs = {}


def build_topo(tgen):
    "Build function"

    def connect_routers(tgen, left_idx, right_idx):
        left = "rt{}".format(left_idx)
        right = "rt{}".format(right_idx)
        switch = tgen.add_switch("s-{}-{}".format(left, right))
        switch.add_link(tgen.gears[left], nodeif="eth-{}".format(right))
        switch.add_link(tgen.gears[right], nodeif="eth-{}".format(left))
        l_addr = "52:54:00:{}:{}:{}".format(left_idx, right_idx, left_idx)
        tgen.gears[left].run("ip link set eth-{} down".format(right))
        tgen.gears[left].run("ip link set eth-{} address {}".format(right, l_addr))
        tgen.gears[left].run("ip link set eth-{} up".format(right))
        r_addr = "52:54:00:{}:{}:{}".format(left_idx, right_idx, right_idx)
        tgen.gears[right].run("ip link set eth-{} down".format(left))
        tgen.gears[right].run("ip link set eth-{} address {}".format(left, r_addr))
        tgen.gears[right].run("ip link set eth-{} up".format(left))

    tgen.add_router("rt1")
    tgen.add_router("rt2")
    tgen.add_router("rt3")
    connect_routers(tgen, 1, 2)
    connect_routers(tgen, 2, 3)
    connect_routers(tgen, 3, 1)

    #
    # Populate multi-dimensional dictionary containing all expected outputs
    #
    number_of_steps = 11
    filenames = [
        "show_mpls_table.ref",
        "show_isis_flex_algo.ref",
    ]
    for rname in ["rt1", "rt2", "rt3"]:
        outputs[rname] = {}
        for step in range(1, number_of_steps + 1):
            outputs[rname][step] = {}
            for filename in filenames:
                # Get snapshots relative to the expected network convergence
                filename_pullpath = "{}/{}/step{}/{}".format(CWD, rname, step, filename)
                outputs[rname][step][filename] = open(filename_pullpath).read()


def setup_module(mod):
    "Sets up the pytest environment"
    tgen = Topogen(build_topo, mod.__name__)
    frrdir = tgen.config.get(tgen.CONFIG_SECTION, "frrdir")
    if not os.path.isfile(os.path.join(frrdir, "pathd")):
        pytest.skip("pathd daemon wasn't built")
    tgen.start_topology()
    router_list = tgen.routers()

    # For all registered routers, load the zebra configuration file
    for rname, router in router_list.items():
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_ISIS, os.path.join(CWD, "{}/isisd.conf".format(rname))
        )
    tgen.start_router()


def teardown_module():
    "Teardown the pytest environment"
    tgen = get_topogen()
    tgen.stop_topology()


def setup_testcase(msg):
    logger.info(msg)
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)
    return tgen


def router_json_cmp_exact_filter(router, cmd, expected):
    output = router.vtysh_cmd(cmd)
    logger.info("{}: {}\n{}".format(router.name, cmd, output))

    json_output = json.loads(output)
    router_output = deepcopy(json_output)

    # filter out dynamic data from "show mpls table"
    for label, data in json_output.items():
        if "1500" in label:
            # filter out SR local labels
            router_output.pop(label)
            continue
        nexthops = data.get("nexthops", [])
        for i in range(len(nexthops)):
            if "fe80::" in nexthops[i].get("nexthop"):
                router_output.get(label).get("nexthops")[i].pop("nexthop")
            elif "." in nexthops[i].get("nexthop"):
                # IPv4, just checking the nexthop
                router_output.get(label).get("nexthops")[i].pop("interface")

    return topotest.json_cmp(router_output, expected, exact=True)


def router_compare_json_output(rname, command, reference):
    "Compare router JSON output"

    logger.info('Comparing router "%s" "%s" output', rname, command)

    tgen = get_topogen()
    expected = json.loads(reference)

    # Run test function until we get an result. Wait at most 60 seconds.
    test_func = partial(
        router_json_cmp_exact_filter, tgen.gears[rname], command, expected
    )
    _, diff = topotest.run_and_expect(test_func, None, count=120, wait=0.5)
    assertmsg = '"{}" JSON output mismatches the expected result'.format(rname)
    assert diff is None, assertmsg


def router_compare_output(rname, command, reference):
    "Compare router output"

    logger.info('Comparing router "%s" "%s" output', rname, command)

    tgen = get_topogen()

    # Run test function until we get an result. Wait at most 60 seconds.
    test_func = partial(
        topotest.router_output_cmp, tgen.gears[rname], command, reference
    )
    result, diff = topotest.run_and_expect(test_func, "", count=120, wait=0.5)
    assertmsg = '{} command "{}" output mismatches the expected result:\n{}'.format(
        rname, command, diff
    )
    assert result, assertmsg


#
# Step 1
#
# Test initial network convergenece
#
# All flex-algo are defined and its fib entries are installed
#
def test_step1_mpls_lfib():
    logger.info("Test (step 1)")
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    # For Developers
    # tgen.mininet_cli()
    for rname in ["rt1", "rt2", "rt3"]:
        router_compare_output(
            rname, "show isis flex-algo", outputs[rname][1]["show_isis_flex_algo.ref"]
        )
        router_compare_json_output(
            rname, "show mpls table json", outputs[rname][1]["show_mpls_table.ref"]
        )


#
# Step 2
#
# Action(s):
# - Disable flex-algo-203 definition advertisement on rt1
#
# Expected change(s):
# - Nothing
#
# Description:
#   No change occurs because it refers to the FAD set in rt2.
#
def test_step2_mpls_lfib():
    logger.info("Test (step 2)")
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    tgen.gears["rt1"].vtysh_cmd(
        """
        configure terminal
         router isis 1
          flex-algo 203
           no advertise-definition
        """
    )

    # For Developers
    # tgen.mininet_cli()
    for rname in ["rt1", "rt2", "rt3"]:
        router_compare_output(
            rname, "show isis flex-algo", outputs[rname][2]["show_isis_flex_algo.ref"]
        )
        router_compare_json_output(
            rname, "show mpls table json", outputs[rname][2]["show_mpls_table.ref"]
        )


#
# Step 3
#
# Action(s):
# - Disable flex-algo-203 definition advertisement on rt2
#
# Expected change(s):
# - rt1,rt2,rt3 should uninstall all Prefix-SIDs of flex-algo-203
#
# Description:
#   When all FADs are disappeared, all their prefix sid routes are withdrawn.
#
def test_step3_mpls_lfib():
    logger.info("Test (step 3)")
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    tgen.gears["rt2"].vtysh_cmd(
        """
        configure terminal
         router isis 1
          flex-algo 203
           no advertise-definition
        """
    )

    # For Developers
    # tgen.mininet_cli()
    for rname in ["rt1", "rt2", "rt3"]:
        router_compare_output(
            rname, "show isis flex-algo", outputs[rname][3]["show_isis_flex_algo.ref"]
        )
        router_compare_json_output(
            rname, "show mpls table json", outputs[rname][3]["show_mpls_table.ref"]
        )


#
# Step 4
#
# Action(s):
# - Enable flex-algo-203 definition advertisement on rt2
#
# Expected change(s):
# - rt1,rt2,rt3 should install all Prefix-SIDs of flex-algo-203
#
# Description:
#   Since the FAD is restored, the reachability to the Prefix-SID is restored.
#
def test_step4_mpls_lfib():
    logger.info("Test (step 4)")
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    tgen.gears["rt2"].vtysh_cmd(
        """
        configure terminal
         router isis 1
          flex-algo 203
           advertise-definition
        """
    )

    # For Developers
    # tgen.mininet_cli()
    for rname in ["rt1", "rt2", "rt3"]:
        router_compare_output(
            rname, "show isis flex-algo", outputs[rname][4]["show_isis_flex_algo.ref"]
        )
        router_compare_json_output(
            rname, "show mpls table json", outputs[rname][4]["show_mpls_table.ref"]
        )


#
# Step 5
#
# Action(s):
# - Enable flex-algo-203 definition advertisement on rt1
#
# Expected change(s):
# - Nothing
#
# Description:
#   This does not affect the FIB, since there is already a FAD for rt2.
#   However, the FAD owner will be changed from rt2 to rt1.
#
def test_step5_mpls_lfib():
    logger.info("Test (step 5)")
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    tgen.gears["rt1"].vtysh_cmd(
        """
        configure terminal
         router isis 1
          flex-algo 203
           advertise-definition
        """
    )

    # For Developers
    # tgen.mininet_cli()
    for rname in ["rt1", "rt2", "rt3"]:
        router_compare_output(
            rname, "show isis flex-algo", outputs[rname][5]["show_isis_flex_algo.ref"]
        )
        router_compare_json_output(
            rname, "show mpls table json", outputs[rname][5]["show_mpls_table.ref"]
        )


#
# Step 6
#
# Action(s):
# - Disable flex-algo-203 SR-MPLS dataplane on rt1
# - Disable flex-algo-203 SR-MPLS dataplane on rt2
# - Disable flex-algo-203 SR-MPLS dataplane on rt3
#
# Expected change(s):
# - rt1,rt2,rt3 should uninstall all Prefix-SIDs of flex-algo-203
#
# Description:
#   Clear the Flex-Algo 203 whole settings on each routers. All routes related
#   to it will be withdrawn.
#
def test_step6_mpls_lfib():
    logger.info("Test (step 6)")
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    for rname in ["rt1", "rt2", "rt3"]:
        tgen.gears[rname].vtysh_cmd(
            """
            configure terminal
             router isis 1
              flex-algo 203
               no dataplane sr-mpls
            """
        )

    # For Developers
    # tgen.mininet_cli()
    for rname in ["rt1", "rt2", "rt3"]:
        router_compare_output(
            rname, "show isis flex-algo", outputs[rname][6]["show_isis_flex_algo.ref"]
        )
        router_compare_json_output(
            rname, "show mpls table json", outputs[rname][6]["show_mpls_table.ref"]
        )


#
# Step 7
#
# Action(s):
# - Disable flex-algo-203 all configuration on rt1
# - Disable flex-algo-203 all configuration on rt2
# - Disable flex-algo-203 all configuration on rt3
#
# Expected change(s):
# - rt1,rt2,rt3 should uninstall all Prefix-SIDs of flex-algo-203
#
# Description:
#   Clear the Flex-Algo 203 whole settings on each routers. All routes related
#   to it will be withdrawn.
#
def test_step7_mpls_lfib():
    logger.info("Test (step 7)")
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    for rname in ["rt1", "rt2", "rt3"]:
        tgen.gears[rname].vtysh_cmd(
            """
            configure terminal
             router isis 1
              no flex-algo 203
            """
        )

    # For Developers
    # tgen.mininet_cli()
    for rname in ["rt1", "rt2", "rt3"]:
        router_compare_output(
            rname, "show isis flex-algo", outputs[rname][7]["show_isis_flex_algo.ref"]
        )
        router_compare_json_output(
            rname, "show mpls table json", outputs[rname][7]["show_mpls_table.ref"]
        )


#
# Step 8
#
# Action(s):
# - Enable flex-algo-203 all configuration on rt1
# - Enable flex-algo-203 all configuration on rt2
# - Enable flex-algo-203 all configuration on rt3
#
# Expected change(s):
# - rt1,rt2,rt3 should install all Prefix-SIDs of flex-algo-203
#
# Description:
#   All configurations were backed.
#
def test_step8_mpls_lfib():
    logger.info("Test (step 8)")
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    tgen.gears["rt1"].vtysh_cmd(
        """
        configure terminal
         router isis 1
          flex-algo 203
           advertise-definition
           affinity exclude-any green
           dataplane sr-mpls
        """
    )

    tgen.gears["rt2"].vtysh_cmd(
        """
        configure terminal
         router isis 1
          flex-algo 203
           advertise-definition
           affinity exclude-any green
           dataplane sr-mpls
        """
    )

    tgen.gears["rt3"].vtysh_cmd(
        """
        configure terminal
         router isis 1
          flex-algo 203
          dataplane sr-mpls
        """
    )

    # For Developers
    # tgen.mininet_cli()
    for rname in ["rt1", "rt2", "rt3"]:
        router_compare_output(
            rname, "show isis flex-algo", outputs[rname][8]["show_isis_flex_algo.ref"]
        )
        router_compare_json_output(
            rname, "show mpls table json", outputs[rname][8]["show_mpls_table.ref"]
        )


#
# Step 9
#
# Action(s):
# - Disable algorithm prefix-sid of algo-203 on rt1
#
# Expected change(s):
# - rt1 should uninstall all Prefix-SIDs of flex-algo-203
# - rt2 should uninstall Prefix-SIDs of rt1's flex-algo-203
# - rt3 should uninstall Prefix-SIDs of rt1's flex-algo-203
#
def test_step9_mpls_lfib():
    logger.info("Test (step 9)")
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    tgen.gears["rt1"].vtysh_cmd(
        """
        configure terminal
         router isis 1
          no segment-routing prefix 1.1.1.1/32 algorithm 203 index 301
          no segment-routing prefix 2001:db8:1000::1/128 algorithm 203 index 1301
        """
    )

    # For Developers
    # tgen.mininet_cli()
    for rname in ["rt1", "rt2", "rt3"]:
        router_compare_output(
            rname, "show isis flex-algo", outputs[rname][9]["show_isis_flex_algo.ref"]
        )
        router_compare_json_output(
            rname, "show mpls table json", outputs[rname][9]["show_mpls_table.ref"]
        )


#
# Step 10
#
# Action(s):
# - Enable algorithm prefix-sid of algo-203 on rt1
#
# Expected change(s):
# - rt1 should install all Prefix-SIDs of flex-algo-203
# - rt2 should install Prefix-SIDs of rt1's flex-algo-203
# - rt3 should install Prefix-SIDs of rt1's flex-algo-203
#
def test_step10_mpls_lfib():
    logger.info("Test (step 10)")
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    tgen.gears["rt1"].vtysh_cmd(
        """
        configure terminal
         router isis 1
          segment-routing prefix 1.1.1.1/32 algorithm 203 index 301
          segment-routing prefix 2001:db8:1000::1/128 algorithm 203 index 1301
        """
    )

    # For Developers
    # tgen.mininet_cli()
    for rname in ["rt1", "rt2", "rt3"]:
        router_compare_output(
            rname, "show isis flex-algo", outputs[rname][10]["show_isis_flex_algo.ref"]
        )
        router_compare_json_output(
            rname, "show mpls table json", outputs[rname][10]["show_mpls_table.ref"]
        )


#
# Step 11
#
# Action(s):
# - Update algorithm prefix-sid of algo-203 on rt1 from 301 to 311
#
# Expected change(s):
# - rt2 should update Prefix-SIDs of rt1's flex-algo-203 from 301 to 311
# - rt3 should update Prefix-SIDs of rt1's flex-algo-203 from 301 to 311
#
def test_step11_mpls_lfib():
    logger.info("Test (step 11)")
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    tgen.gears["rt1"].vtysh_cmd(
        """
        configure terminal
         router isis 1
          segment-routing prefix 1.1.1.1/32 algorithm 203 index 311
          segment-routing prefix 2001:db8:1000::1/128 algorithm 203 index 1311
        """
    )

    # For Developers
    # tgen.mininet_cli()
    for rname in ["rt1", "rt2", "rt3"]:
        router_compare_output(
            rname, "show isis flex-algo", outputs[rname][11]["show_isis_flex_algo.ref"]
        )
        router_compare_json_output(
            rname, "show mpls table json", outputs[rname][11]["show_mpls_table.ref"]
        )


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python3

"""
exa-receive.py: Save received routes form ExaBGP into file
"""

import argparse
import os
from sys import stdin
from datetime import datetime

parser = argparse.ArgumentParser()
parser.add_argument(
    "--no-timestamp", dest="timestamp", action="store_false", help="Disable timestamps"
)
parser.add_argument(
    "--logdir", default="/tmp/gearlogdir", help="The directory to store the peer log in"
)
parser.add_argument("peer", type=int, help="The peer number")
args = parser.parse_args()

savepath = os.path.join(args.logdir, "peer{}-received.log".format(args.peer))
routesavefile = open(savepath, "w")

while True:
    try:
        line = stdin.readline()
        if not line:
            break

        if not args.timestamp:
            routesavefile.write(line)
        else:
            timestamp = datetime.now().strftime("%Y%m%d_%H:%M:%S - ")
            routesavefile.write(timestamp + line)
        routesavefile.flush()
    except KeyboardInterrupt:
        pass
    except IOError:
        # most likely a signal during readline
        pass

routesavefile.close()
